@@ -4,7 +4,7 @@
 
 import os
 
-version = "0.3.7"
+version = "0.4.1"
 location = os.path.abspath(os.path.dirname(__file__))
 
 
@@ -50,6 +50,7 @@ def get_scripts(top, name):
 	for loc in locations:
 		if any(os.path.dirname(loc).endswith(t) for t in locations):
 			continue
-		scripts.append(loc)
+		scripts.append(loc.replace('\\', '/'))
 	return scripts
 
+
