@@ -7,7 +7,7 @@ Summary
 -------
 Exports and converts *waf* project data, for C/C++ programs, static- and shared
 libraries, into **Microsoft Visual Studio**, also known as **msdev**, 
-project files (.vcproj) and solution (*.sln) files.
+project files (.vcproj) and solution (.sln) files.
 
 **Microsoft Visual Studio** is a mature and stable integrated development 
 environment for, amongst others, the C and C++ programming language. A free
@@ -58,12 +58,12 @@ environment as much as possible. Note that since cross compilation is not
 really supported in this IDE, only the first environment encountered that
 is targeted for **MS Windows** will be exported; i.e. an environment in 
 which::
-	
-	bld.env.DEST_OS == 'win32'
+
+    bld.env.DEST_OS == 'win32'
 
 is true.
 
-	
+
 Please note that in contrast to a *normal* IDE setup the exported projects 
 will contain either a *debug* **or** a *release* build target but not both at
 the same time. By doing so exported projects will always use the same settings
@@ -76,15 +76,22 @@ Usage
 **Visual Studio** project and workspace files can be exported using the *msdev* 
 command, as shown in the example below::
 
-        $ waf msdev
+    $ waf msdev
 
 When needed, exported **Visual Studio** project- and solution files can be 
 removed using the *clean* command, as shown in the example below::
 
-        $ waf msdev --clean
+    $ waf msdev --clean
 
 Once exported simply open the *appname.sln* using **Visual Studio**
 this will automatically open all exported projects as well.
+
+Tasks generators to be excluded can be marked with the *skipme* option 
+as shown below::
+
+    def build(bld):
+        bld.program(name='foo', src='foobar.c', msdev_skip=True)
+
 '''
 
 import os
@@ -96,6 +103,9 @@ import xml.etree.ElementTree as ElementTree
 from xml.dom import minidom
 from waflib import Utils, Logs, Errors, Context
 from waflib.Build import BuildContext
+import waftools
+from waftools import deps
+
 
 def options(opt):
 	'''Adds command line options to the *waf* build environment 
@@ -162,7 +172,13 @@ def export(bld):
 		return
 
 	solution = MsDevSolution(bld)
+	targets = waftools.deps.get_targets(bld)
+
 	for tgen in bld.task_gen_cache_names.values():
+		if targets and tgen.get_name() not in targets:
+			continue
+		if getattr(tgen, 'msdev_skipme', False):
+			continue
 		if set(('c', 'cxx')) & set(getattr(tgen, 'features', [])):
 			project = MsDevProject(bld, tgen)
 			project.export()
@@ -183,9 +199,16 @@ def cleanup(bld):
 	if not bld.options.msdev and not hasattr(bld, 'msdev'):
 		return
 
+	targets = waftools.deps.get_targets(bld)
+
 	for tgen in bld.task_gen_cache_names.values():
-		project = MsDevProject(bld, tgen)
-		project.cleanup()
+		if targets and tgen.get_name() not in targets:
+			continue
+		if getattr(tgen, 'msdev_skipme', False):
+			continue
+		if set(('c', 'cxx')) & set(getattr(tgen, 'features', [])):
+			project = MsDevProject(bld, tgen)
+			project.cleanup()
 
 	solution = MsDevSolution(bld)
 	solution.cleanup()
@@ -196,11 +219,11 @@ class MsDev(object):
 	**Visual Studio** projects and solutions.
 
 	:param bld: Build context as used in *wscript* files of your *waf* build
-				environment.
-	:type bld:	waflib.Build.BuildContext
+	            environment.
+	:type bld:  waflib.Build.BuildContext
 	'''
 
-	PROGRAM	= '1'
+	PROGRAM = '1'
 	'''Identifier for projects containing an executable'''
 
 	OBJECT  = '3'
@@ -245,7 +268,7 @@ class MsDev(object):
 			Logs.pprint('YELLOW', 'removed: %s' % node.abspath())
 		for node in cwd.ant_glob('*.sln'):
 			node.delete()
-			Logs.pprint('YELLOW', 'removed: %s' % node.abspath())			
+			Logs.pprint('YELLOW', 'removed: %s' % node.abspath())
 		node = self._find_node()
 		if node:
 			node.delete()
@@ -290,8 +313,8 @@ class MsDevSolution(MsDev):
 	environment.
 
 	:param bld: Build context as used in *wscript* files of your *waf* build
-				environment.
-	:type bld:	waflib.Build.BuildContext
+	            environment.
+	:type bld:  waflib.Build.BuildContext
 	'''
 	
 	def __init__(self, bld):
@@ -304,7 +327,7 @@ class MsDevSolution(MsDev):
 		return '%s.sln' % getattr(Context.g_module, Context.APPNAME)
 
 	def export(self):
-		'''Exports a **Visual Studio** solution.'''	
+		'''Exports a **Visual Studio** solution.'''
 		src = '%s/msdev.sln' % os.path.dirname(__file__)
 		dst = self._get_fname()
 		shutil.copyfile(src, dst)
@@ -326,7 +349,7 @@ class MsDevSolution(MsDev):
 						except KeyError:
 							pass
 						else:
-							f.write('\t\t{%s} = {%s}\n' % (pid, pid))						
+							f.write('\t\t{%s} = {%s}\n' % (pid, pid))
 					f.write('\tEndProjectSection\n')
 				f.write('EndProject\n')
 			for line in s[3:8]:
@@ -340,12 +363,12 @@ class MsDevSolution(MsDev):
 	def add_project(self, name, fname, deps, pid):
 		'''Adds a project to the workspace.
 		
-		:param name:	Name of the project.
-		:type name:		str
-		:param fname:	Complete path to the project file
-		:type fname: 	str
-		:param deps:	List of names on which this project depends
-		:type deps: 	list of str
+		:param name:    Name of the project.
+		:type name:     str
+		:param fname:   Complete path to the project file
+		:type fname:    str
+		:param deps:    List of names on which this project depends
+		:type deps:     list of str
 		'''
 		self.projects[name] = (fname, deps, pid)
 
@@ -355,12 +378,12 @@ class MsDevProject(MsDev):
 	projects.
 
 	:param bld: Build context as used in *wscript* files of your *waf* build
-				environment.
-	:type bld:	waflib.Build.BuildContext
+	            environment.
+	:type bld:  waflib.Build.BuildContext
 	
 	:param gen: Task generator that contains all information of the task to be
-				converted and exported to the **Visual Studio** project.
-	:type gen:	waflib.Task.TaskGen	
+	            converted and exported to the **Visual Studio** project.
+	:type gen:  waflib.Task.TaskGen
 	'''
 
 	def __init__(self, bld, gen):
@@ -451,7 +474,7 @@ class MsDevProject(MsDev):
 			if filtr.get('Name') == 'Source Files':
 				for source in filtr.iter('File'):
 					sources.append(source.get('RelativePath'))
-				break		
+				break
 		if len(sources) == 0:
 			filtr = ElementTree.SubElement(files, 'Filter', attrib={'Name':'Source Files', 'Filter':'cpp;c;cc;cxx;def;odl;idl;hpj;bat;asm;asmx'})
 			filtr.set('UniqueIdentifier', '{%s}' % str(uuid.uuid4()).upper())
@@ -560,8 +583,8 @@ class MsDevProject(MsDev):
 		gen = self.gen
 		defines = self._get_genlist(gen, 'defines') + gen.bld.env.DEFINES
 		if 'win32' in sys.platform:
-			defines = [d.replace('"', '\\"') for d in defines]			
-		defines = ';'.join(defines)			
+			defines = [d.replace('"', '\\"') for d in defines]
+		defines = ';'.join(defines)
 		return defines
 
 	def _get_link_options(self):
@@ -581,7 +604,7 @@ class MsDevProject(MsDev):
 		libs = Utils.to_list(getattr(gen, 'lib', []))
 		for l in ('m'): # remove posix libraries, that integrated in MVSCRT
 			if libs.count(l):
-				libs.remove(l)		
+				libs.remove(l)
 		deps = Utils.to_list(getattr(gen, 'use', []))
 		for dep in deps:
 			try:
@@ -717,3 +740,4 @@ MSDEV_PROJECT = \
 	</Globals>
 </VisualStudioProject>
 '''
+
